@@ -19,7 +19,6 @@ import (
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/runtimeclass"
 	"github.com/openshift-kni/performance-addon-operators/pkg/controller/performanceprofile/components/tuned"
 	testutils "github.com/openshift-kni/performance-addon-operators/pkg/utils/testing"
-	v1 "github.com/openshift/api/config/v1"
 	tunedv1 "github.com/openshift/cluster-node-tuning-operator/pkg/apis/tuned/v1"
 	conditionsv1 "github.com/openshift/custom-resource-status/conditions/v1"
 	mcov1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
@@ -40,7 +39,6 @@ import (
 )
 
 const assetsDir = "../build/assets"
-const clusterVersion = "4.7.12"
 
 var _ = Describe("Controller", func() {
 	var request reconcile.Request
@@ -75,49 +73,13 @@ var _ = Describe("Controller", func() {
 			profile.Finalizers = append(profile.Finalizers, finalizer)
 		})
 
-		It("should validate scripts required parameters", func() {
-			profile.Spec.MachineConfigPoolSelector = map[string]string{"fake1": "val1", "fake2": "val2"}
-			r := newFakeReconciler(profile)
-
-			// we do not return error, because we do not want to reconcile again, and just print error under the log,
-			// once we will have validation webhook, this test will not be relevant anymore
-			Expect(reconcileTimes(r, request, 1)).To(Equal(reconcile.Result{}))
-
-			updatedProfile := &performancev2.PerformanceProfile{}
-			key := types.NamespacedName{
-				Name:      profile.Name,
-				Namespace: metav1.NamespaceNone,
-			}
-			Expect(r.Get(context.TODO(), key, updatedProfile)).ToNot(HaveOccurred())
-
-			// verify profile conditions
-			degradeCondition := conditionsv1.FindStatusCondition(updatedProfile.Status.Conditions, conditionsv1.ConditionDegraded)
-			Expect(degradeCondition).ToNot(BeNil())
-			Expect(degradeCondition.Status).To(Equal(corev1.ConditionTrue))
-			Expect(degradeCondition.Reason).To(Equal(conditionReasonValidationFailed))
-			Expect(degradeCondition.Message).To(Equal("validation error: you should provide only 1 MachineConfigPoolSelector"))
-
-			// verify validation event
-			fakeRecorder, ok := r.Recorder.(*record.FakeRecorder)
-			Expect(ok).To(BeTrue())
-			event := <-fakeRecorder.Events
-			Expect(event).To(ContainSubstring("Validation failed"))
-
-			// verify that no components created by the controller
-			mcp := &mcov1.MachineConfigPool{}
-			key.Name = components.GetComponentName(profile.Name, components.ComponentNamePrefix)
-			err := r.Get(context.TODO(), key, mcp)
-			Expect(errors.IsNotFound(err)).To(Equal(true))
-		})
-
 		It("should create all resources on first reconcile loop", func() {
 			r := newFakeReconciler(profile)
 
 			Expect(reconcileTimes(r, request, 1)).To(Equal(reconcile.Result{}))
 
-			name := components.GetComponentName(profile.Name, components.ComponentNamePrefix)
 			key := types.NamespacedName{
-				Name:      name,
+				Name:      machineconfig.GetMachineConfigName(profile),
 				Namespace: metav1.NamespaceNone,
 			}
 
@@ -126,6 +88,11 @@ var _ = Describe("Controller", func() {
 			err := r.Get(context.TODO(), key, mc)
 			Expect(err).ToNot(HaveOccurred())
 
+			key = types.NamespacedName{
+				Name:      components.GetComponentName(profile.Name, components.ComponentNamePrefix),
+				Namespace: metav1.NamespaceNone,
+			}
+
 			// verify KubeletConfig creation
 			kc := &mcov1.KubeletConfig{}
 			err = r.Get(context.TODO(), key, kc)
@@ -276,13 +243,13 @@ var _ = Describe("Controller", func() {
 
 		It("should remove outdated tuned objects", func() {
 
-			tunedOutdatedA, err := tuned.NewNodePerformance(assetsDir, profile, clusterVersion)
+			tunedOutdatedA, err := tuned.NewNodePerformance(assetsDir, profile)
 			Expect(err).ToNot(HaveOccurred())
 			tunedOutdatedA.Name = "outdated-a"
 			tunedOutdatedA.OwnerReferences = []metav1.OwnerReference{
 				{Name: profile.Name},
 			}
-			tunedOutdatedB, err := tuned.NewNodePerformance(assetsDir, profile, clusterVersion)
+			tunedOutdatedB, err := tuned.NewNodePerformance(assetsDir, profile)
 			Expect(err).ToNot(HaveOccurred())
 			tunedOutdatedB.Name = "outdated-b"
 			tunedOutdatedB.OwnerReferences = []metav1.OwnerReference{
@@ -372,7 +339,7 @@ var _ = Describe("Controller", func() {
 				kc, err = kubeletconfig.New(profile)
 				Expect(err).ToNot(HaveOccurred())
 
-				tunedPerformance, err = tuned.NewNodePerformance(assetsDir, profile, clusterVersion)
+				tunedPerformance, err = tuned.NewNodePerformance(assetsDir, profile)
 				Expect(err).ToNot(HaveOccurred())
 
 				runtimeClass = runtimeclass.New(profile, machineconfig.HighPerformanceRuntime)
@@ -400,9 +367,8 @@ var _ = Describe("Controller", func() {
 
 				Expect(reconcileTimes(r, request, 1)).To(Equal(reconcile.Result{}))
 
-				name := components.GetComponentName(profile.Name, components.ComponentNamePrefix)
 				key := types.NamespacedName{
-					Name:      name,
+					Name:      machineconfig.GetMachineConfigName(profile),
 					Namespace: metav1.NamespaceNone,
 				}
 
@@ -554,7 +520,7 @@ var _ = Describe("Controller", func() {
 
 				By("Verifying MC update")
 				key = types.NamespacedName{
-					Name:      components.GetComponentName(profile.Name, components.ComponentNamePrefix),
+					Name:      machineconfig.GetMachineConfigName(profile),
 					Namespace: metav1.NamespaceNone,
 				}
 				mc := &mcov1.MachineConfig{}
@@ -675,6 +641,72 @@ var _ = Describe("Controller", func() {
 				Expect(degradedCondition.Reason).To(Equal(conditionReasonMCPDegraded))
 				Expect(degradedCondition.Message).To(ContainSubstring(mcpMessage))
 			})
+
+			It("should update status when TunedProfile is degraded", func() {
+				tunedReason := "tunedReason"
+				tunedMessage := "Tuned message"
+
+				tuned := &tunedv1.Profile{
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "tuned-profile-test",
+					},
+					Status: tunedv1.ProfileStatus{
+						Conditions: []tunedv1.ProfileStatusCondition{
+							{
+								Type:    tunedv1.TunedDegraded,
+								Status:  corev1.ConditionTrue,
+								Reason:  tunedReason,
+								Message: tunedMessage,
+							},
+							{
+								Type:    tunedv1.TunedProfileApplied,
+								Status:  corev1.ConditionFalse,
+								Reason:  tunedReason,
+								Message: tunedMessage,
+							},
+						},
+					},
+				}
+
+				nodes := &corev1.NodeList{
+					Items: []corev1.Node{
+						{
+							ObjectMeta: metav1.ObjectMeta{
+								Name: "tuned-profile-test",
+								Labels: map[string]string{
+									"nodekey": "nodeValue",
+								},
+							},
+						},
+						{
+							ObjectMeta: metav1.ObjectMeta{
+								Name: "tuned-profile-test2",
+							},
+						},
+					},
+				}
+
+				r := newFakeReconciler(profile, mc, kc, tunedPerformance, tuned, nodes)
+
+				Expect(reconcileTimes(r, request, 1)).To(Equal(reconcile.Result{}))
+
+				updatedProfile := &performancev2.PerformanceProfile{}
+				key := types.NamespacedName{
+					Name:      profile.Name,
+					Namespace: metav1.NamespaceNone,
+				}
+				Expect(r.Get(context.TODO(), key, updatedProfile)).ToNot(HaveOccurred())
+
+				// verify performance profile status
+				Expect(len(updatedProfile.Status.Conditions)).To(Equal(4))
+
+				// verify profile conditions
+				degradedCondition := conditionsv1.FindStatusCondition(updatedProfile.Status.Conditions, conditionsv1.ConditionDegraded)
+				Expect(degradedCondition).ToNot(BeNil())
+				Expect(degradedCondition.Status).To(Equal(corev1.ConditionTrue))
+				Expect(degradedCondition.Reason).To(Equal(conditionReasonTunedDegraded))
+				Expect(degradedCondition.Message).To(ContainSubstring(tunedMessage))
+			})
 		})
 
 	})
@@ -695,7 +727,7 @@ var _ = Describe("Controller", func() {
 			kc, err := kubeletconfig.New(profile)
 			Expect(err).ToNot(HaveOccurred())
 
-			tunedPerformance, err := tuned.NewNodePerformance(assetsDir, profile, clusterVersion)
+			tunedPerformance, err := tuned.NewNodePerformance(assetsDir, profile)
 			Expect(err).ToNot(HaveOccurred())
 
 			runtimeClass := runtimeclass.New(profile, machineconfig.HighPerformanceRuntime)
@@ -779,20 +811,7 @@ func reconcileTimes(reconciler *PerformanceProfileReconciler, request reconcile.
 
 // newFakeReconciler returns a new reconcile.Reconciler with a fake client
 func newFakeReconciler(initObjects ...runtime.Object) *PerformanceProfileReconciler {
-	cv := &v1.ClusterVersion{
-		TypeMeta: metav1.TypeMeta{
-			APIVersion: "config.openshift.io/v1",
-			Kind:       "MachineConfigPool",
-		},
-		ObjectMeta: metav1.ObjectMeta{
-			Name: "version",
-		},
-		Status: v1.ClusterVersionStatus{
-			Desired: v1.Update{Version: "4.7.12"},
-		},
-	}
-	fakeObjects := append(initObjects, cv)
-	fakeClient := fake.NewFakeClientWithScheme(scheme.Scheme, fakeObjects...)
+	fakeClient := fake.NewFakeClientWithScheme(scheme.Scheme, initObjects...)
 	fakeRecorder := record.NewFakeRecorder(10)
 	return &PerformanceProfileReconciler{
 		Client:    fakeClient,
