@@ -11,16 +11,14 @@ import (
 	"strings"
 	"time"
 
-	"github.com/hashicorp/go-version"
 	. "github.com/onsi/ginkgo"
 	. "github.com/onsi/gomega"
-	. "github.com/onsi/gomega/gstruct"
 
 	corev1 "k8s.io/api/core/v1"
 	"k8s.io/api/node/v1beta1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/fields"
-	"k8s.io/apimachinery/pkg/labels"
+	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/klog"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
@@ -28,7 +26,6 @@ import (
 
 	"sigs.k8s.io/controller-runtime/pkg/client"
 
-	v1 "github.com/openshift/api/config/v1"
 	tunedv1 "github.com/openshift/cluster-node-tuning-operator/pkg/apis/tuned/v1"
 	machineconfigv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 
@@ -37,7 +34,9 @@ import (
 	performancev2 "github.com/openshift-kni/performance-addon-operators/api/v2"
 	testutils "github.com/openshift-kni/performance-addon-operators/functests/utils"
 	testclient "github.com/openshift-kni/performance-addon-operators/functests/utils/client"
+	"github.com/openshift-kni/performance-addon-operators/functests/utils/cluster"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/discovery"
+	testlog "github.com/openshift-kni/performance-addon-operators/functests/utils/log"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/mcps"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/nodes"
 	"github.com/openshift-kni/performance-addon-operators/functests/utils/pods"
@@ -52,11 +51,19 @@ const (
 	testPollInterval = 2
 )
 
+var RunningOnSingleNode bool
+
 var _ = Describe("[rfe_id:27368][performance]", func() {
 
 	var workerRTNodes []corev1.Node
 	var profile *performancev2.PerformanceProfile
 
+	testutils.BeforeAll(func() {
+		isSNO, err := cluster.IsSingleNode()
+		Expect(err).ToNot(HaveOccurred())
+		RunningOnSingleNode = isSNO
+	})
+
 	BeforeEach(func() {
 		if discovery.Enabled() && testutils.ProfileNotFound {
 			Skip("Discovery mode enabled, performance profile not found")
@@ -64,18 +71,18 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 
 		var err error
 		workerRTNodes, err = nodes.GetByLabels(testutils.NodeSelectorLabels)
-		Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("error looking for node with role %q: %v", testutils.RoleWorkerCNF, err))
+		Expect(err).ToNot(HaveOccurred(), "error looking for node with role %q: %v", testutils.RoleWorkerCNF, err)
 		workerRTNodes, err = nodes.MatchingOptionalSelector(workerRTNodes)
 		Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("error looking for the optional selector: %v", err))
-		Expect(workerRTNodes).ToNot(BeEmpty(), fmt.Sprintf("no nodes with role %q found", testutils.RoleWorkerCNF))
+		Expect(workerRTNodes).ToNot(BeEmpty(), "no nodes with role %q found", testutils.RoleWorkerCNF)
 		profile, err = profiles.GetByNodeLabels(testutils.NodeSelectorLabels)
-		Expect(err).ToNot(HaveOccurred())
+		Expect(err).ToNot(HaveOccurred(), "cannot get profile by node labels %v", testutils.NodeSelectorLabels)
 	})
 
 	// self-tests; these are only vaguely related to performance becase these are enablement conditions, not actual settings.
 	// For example, running on control plane means we leave more resources for the workload.
 	Context("Performance Operator", func() {
-		It("Should run on the control plane nodes", func() {
+		It("[test_id:38109] Should run on the control plane nodes", func() {
 			pod, err := pods.GetPerformanceOperatorPod()
 			Expect(err).ToNot(HaveOccurred(), "Failed to find the Performance Addon Operator pod")
 
@@ -90,8 +97,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 				}
 			}
 
-			// Fail
-			Expect(true).To(Reject(), "Performance Addon Operator is not running in a master node")
+			Fail("Performance Addon Operator is not running in a master node")
 		})
 	})
 
@@ -105,7 +111,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 			tuned := &tunedv1.Tuned{}
 			err := testclient.Client.Get(context.TODO(), key, tuned)
-			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object "+tuned.Name)
+			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object %q", tuned.Name)
 
 			Eventually(func() bool {
 				err := testclient.Client.List(context.TODO(), tunedList)
@@ -120,16 +126,17 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 					}
 				}
 				return true
-			}, 120*time.Second, testPollInterval*time.Second).Should(BeTrue(),
-				"tuned CR name owned by a performance profile CR should only be "+tunedExpectedName)
+			}, cluster.ComputeTestTimeout(120*time.Second, RunningOnSingleNode), testPollInterval*time.Second).Should(BeTrue(),
+				"tuned CR name owned by a performance profile CR should only be %q", tunedExpectedName)
 		})
 
 		It("[test_id:37127] Node should point to right tuned profile", func() {
 			for _, node := range workerRTNodes {
-				tuned := tunedForNode(&node)
+				tuned := nodes.TunedForNode(&node, RunningOnSingleNode)
 				activeProfile, err := pods.ExecCommandOnPod(tuned, []string{"cat", "/etc/tuned/active_profile"})
-				Expect(err).ToNot(HaveOccurred())
-				Expect(strings.TrimSpace(string(activeProfile))).To(Equal(tunedExpectedName))
+				Expect(err).ToNot(HaveOccurred(), "Error getting the tuned active profile")
+				activeProfileName := string(activeProfile)
+				Expect(strings.TrimSpace(activeProfileName)).To(Equal(tunedExpectedName), "active profile name mismatch got %q expected %q", activeProfileName, tunedExpectedName)
 			}
 		})
 	})
@@ -145,7 +152,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 		})
 
-		It("Should set CPU isolcpu's kernel argument managed_irq flag", func() {
+		It("[test_id:32702] Should set CPU isolcpu's kernel argument managed_irq flag", func() {
 			for _, node := range workerRTNodes {
 				cmdline, err := nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"cat", "/proc/cmdline"})
 				Expect(err).ToNot(HaveOccurred())
@@ -196,57 +203,64 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 		})
 
 		It("[test_id:35363][crit:high][vendor:cnf-qe@redhat.com][level:acceptance] stalld daemon is running on the host", func() {
-			By("Checking minimal required cluster version")
-			clusterVersion := ""
-			cv := &v1.ClusterVersion{}
-			key := types.NamespacedName{
-				Name: "version",
-			}
-			err := testclient.Client.Get(context.TODO(), key, cv)
-			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster version object "+key.String())
-			clusterVersion = cv.Status.Desired.Version
-			currentClusterVersion, err := version.NewVersion(clusterVersion)
-			Expect(err).ToNot(HaveOccurred())
-			requiredStalldClusterVersion, err := version.NewVersion("4.7.7")
-			Expect(err).ToNot(HaveOccurred())
-			if strings.Contains(clusterVersion, "ci") || strings.Contains(clusterVersion, "nightly") ||
-				currentClusterVersion.GreaterThanOrEqual(requiredStalldClusterVersion) {
-				for _, node := range workerRTNodes {
-					tuned := tunedForNode(&node)
-					_, err = pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
-					Expect(err).ToNot(HaveOccurred())
-				}
-			} else {
-				klog.Warningf("Stalld is not enable, cluster version: %s", clusterVersion)
-			}
-
-			// this test will both determine if the minimum required kernel version exists in order for stalld to work
-			// and alert in case a wrong kernel version is provided in an openshift release that is being tested.
-			By("Checking minmal required kernel version")
-			stalldMinimumKernelVersionPrefix := "4.18.0-240.22"
-			minKernelPrefix := strings.Split(stalldMinimumKernelVersionPrefix, ".")
-			minSubVersion, err := strconv.Atoi(minKernelPrefix[3])
-			Expect(err).ToNot(HaveOccurred())
 			for _, node := range workerRTNodes {
-				cmd := []string{"uname", "-r"}
-				kernel, err := nodes.ExecCommandOnNode(cmd, &node)
-				Expect(err).ToNot(HaveOccurred(), "failed to execute uname")
-				currentKernelPrefix := strings.Split(kernel, ".")
-				currentSubVersion, err := strconv.Atoi(currentKernelPrefix[3])
-				// check if current kernel is a new rt batch stream (higher than 0-240)
-				if err != nil && strings.Contains(currentKernelPrefix[3], "rt") {
-					batchVersionStr := strings.Split(currentKernelPrefix[2], "-")
-					batchVersion, err := strconv.Atoi(batchVersionStr[1])
-					Expect(err).ToNot(HaveOccurred())
-					Expect(batchVersion > 240).Should(BeTrue(), fmt.Sprintf("Current kernel version %s is incompatible to the current oc release", kernel))
-					tuned := tunedForNode(&node)
-					_, err = pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
-					Expect(err).ToNot(HaveOccurred())
-				} else if currentSubVersion >= minSubVersion {
-					tuned := tunedForNode(&node)
-					_, err := pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
-					Expect(err).ToNot(HaveOccurred())
+				tuned := nodes.TunedForNode(&node, RunningOnSingleNode)
+				_, err := pods.ExecCommandOnPod(tuned, []string{"pidof", "stalld"})
+				Expect(err).ToNot(HaveOccurred())
+			}
+		})
+		It("[test_id:42400][crit:medium][vendor:cnf-qe@redhat.com][level:acceptance] stalld daemon is running as sched_fifo", func() {
+			for _, node := range workerRTNodes {
+				pid, err := nodes.ExecCommandOnNode([]string{"pidof", "stalld"}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(pid).ToNot(BeEmpty())
+				sched_tasks, err := nodes.ExecCommandOnNode([]string{"chrt", "-ap", pid}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(sched_tasks).To(ContainSubstring("scheduling policy: SCHED_FIFO"))
+				Expect(sched_tasks).To(ContainSubstring("scheduling priority: 10"))
+			}
+		})
+		It("[test_id:42696][crit:medium][vendor:cnf-qe@redhat.com][level:acceptance] Stalld runs in higher priority than ksoftirq and rcu{c,b}", func() {
+			for _, node := range workerRTNodes {
+				stalld_pid, err := nodes.ExecCommandOnNode([]string{"pidof", "stalld"}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(stalld_pid).ToNot(BeEmpty())
+				sched_tasks, err := nodes.ExecCommandOnNode([]string{"chrt", "-ap", stalld_pid}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				re := regexp.MustCompile("scheduling priority: ([0-9]+)")
+				match := re.FindStringSubmatch(sched_tasks)
+				stalld_prio, err := strconv.Atoi(match[1])
+				Expect(err).ToNot(HaveOccurred())
+
+				ksoftirq_pid, err := nodes.ExecCommandOnNode([]string{"pgrep", "-f", "ksoftirqd", "-n"}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(ksoftirq_pid).ToNot(BeEmpty())
+				sched_tasks, err = nodes.ExecCommandOnNode([]string{"chrt", "-ap", ksoftirq_pid}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				match = re.FindStringSubmatch(sched_tasks)
+				ksoftirq_prio, err := strconv.Atoi(match[1])
+				Expect(err).ToNot(HaveOccurred())
+
+				if profile.Spec.RealTimeKernel == nil ||
+					profile.Spec.RealTimeKernel.Enabled == nil ||
+					*profile.Spec.RealTimeKernel.Enabled != true {
+					Expect(stalld_prio).To(BeNumerically("<", ksoftirq_prio))
+					testlog.Warning("Skip checking rcu since RT kernel is disabled")
+					return
 				}
+				//rcuc/n : kthreads that are pinned to CPUs & are responsible to execute the callbacks of rcu threads .
+				//rcub/n : are boosting kthreads ,responsible to monitor per-cpu arrays of lists of tasks that were blocked while in an rcu read-side critical sections.
+				rcu_pid, err := nodes.ExecCommandOnNode([]string{"pgrep", "-f", "rcu[c,b]", "-n"}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				Expect(rcu_pid).ToNot(BeEmpty())
+				sched_tasks, err = nodes.ExecCommandOnNode([]string{"chrt", "-ap", rcu_pid}, &node)
+				Expect(err).ToNot(HaveOccurred())
+				match = re.FindStringSubmatch(sched_tasks)
+				rcu_prio, err := strconv.Atoi(match[1])
+				Expect(err).ToNot(HaveOccurred())
+
+				Expect(stalld_prio).To(BeNumerically("<", rcu_prio))
+				Expect(stalld_prio).To(BeNumerically("<", ksoftirq_prio))
 			}
 		})
 
@@ -283,7 +297,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			tuned := &tunedv1.Tuned{}
 			err := testclient.Client.Get(context.TODO(), key, tuned)
 			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object "+key.String())
-			validatTunedActiveProfile(workerRTNodes)
+			validateTunedActiveProfile(workerRTNodes)
 			execSysctlOnWorkers(workerRTNodes, sysctlMap)
 		})
 	})
@@ -309,7 +323,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 				hook := hooks["hook"].(map[string]interface{})
 				Expect(hook).ToNot(BeNil())
 				args := hook["args"].([]interface{})
-				Expect(len(args)).To(Equal(2))
+				Expect(len(args)).To(Equal(2), "unexpected arguments: %v", args)
 
 				rpsCPUs, err := components.CPUMaskToCPUSet(args[1].(string))
 				Expect(err).ToNot(HaveOccurred())
@@ -374,7 +388,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			tuned := &tunedv1.Tuned{}
 			err := testclient.Client.Get(context.TODO(), key, tuned)
 			Expect(err).ToNot(HaveOccurred(), "cannot find the Cluster Node Tuning Operator object "+components.ProfileNamePerformance)
-			validatTunedActiveProfile(workerRTNodes)
+			validateTunedActiveProfile(workerRTNodes)
 			execSysctlOnWorkers(workerRTNodes, sysctlMap)
 		})
 	})
@@ -516,7 +530,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			By("Checking that new KubeletConfig, MachineConfig and RuntimeClass created")
 			configKey := types.NamespacedName{
 				Name:      components.GetComponentName(secondProfile.Name, components.ComponentNamePrefix),
-				Namespace: components.NamespaceNodeTuningOperator,
+				Namespace: metav1.NamespaceNone,
 			}
 			kubeletConfig := &machineconfigv1.KubeletConfig{}
 			err := testclient.GetWithRetry(context.TODO(), configKey, kubeletConfig)
@@ -524,16 +538,20 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			Expect(kubeletConfig.Spec.MachineConfigPoolSelector.MatchLabels[machineconfigv1.MachineConfigRoleLabelKey]).Should(Equal(newRole))
 			Expect(kubeletConfig.Spec.KubeletConfig.Raw).Should(ContainSubstring("restricted"), "Can't find value in KubeletConfig")
 
-			machineConfig := &machineconfigv1.MachineConfig{}
-			err = testclient.GetWithRetry(context.TODO(), configKey, machineConfig)
-			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find MachineConfig object %s", configKey.Name))
-			Expect(machineConfig.Labels[machineconfigv1.MachineConfigRoleLabelKey]).Should(Equal(newRole))
-
 			runtimeClass := &v1beta1.RuntimeClass{}
 			err = testclient.GetWithRetry(context.TODO(), configKey, runtimeClass)
 			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find RuntimeClass profile object %s", runtimeClass.Name))
 			Expect(runtimeClass.Handler).Should(Equal(machineconfig.HighPerformanceRuntime))
 
+			machineConfigKey := types.NamespacedName{
+				Name:      machineconfig.GetMachineConfigName(secondProfile),
+				Namespace: metav1.NamespaceNone,
+			}
+			machineConfig := &machineconfigv1.MachineConfig{}
+			err = testclient.GetWithRetry(context.TODO(), machineConfigKey, machineConfig)
+			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find MachineConfig object %s", configKey.Name))
+			Expect(machineConfig.Labels[machineconfigv1.MachineConfigRoleLabelKey]).Should(Equal(newRole))
+
 			By("Checking that new Tuned profile created")
 			tunedKey := types.NamespacedName{
 				Name:      components.GetComponentName(secondProfile.Name, components.ProfileNamePerformance),
@@ -564,7 +582,7 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}, 30, 5).Should(Equal(corev1.ConditionFalse))
 
 			Expect(testclient.Client.Get(context.TODO(), configKey, kubeletConfig)).To(HaveOccurred(), fmt.Sprintf("KubeletConfig %s should be removed", configKey.Name))
-			Expect(testclient.Client.Get(context.TODO(), configKey, machineConfig)).To(HaveOccurred(), fmt.Sprintf("MachineConfig %s should be removed", configKey.Name))
+			Expect(testclient.Client.Get(context.TODO(), machineConfigKey, machineConfig)).To(HaveOccurred(), fmt.Sprintf("MachineConfig %s should be removed", configKey.Name))
 			Expect(testclient.Client.Get(context.TODO(), configKey, runtimeClass)).To(HaveOccurred(), fmt.Sprintf("RuntimeClass %s should be removed", configKey.Name))
 			Expect(testclient.Client.Get(context.TODO(), tunedKey, tunedProfile)).To(HaveOccurred(), fmt.Sprintf("Tuned profile object %s should be removed", tunedKey.Name))
 
@@ -575,7 +593,12 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			}
 			err = testclient.GetWithRetry(context.TODO(), initialKey, &machineconfigv1.KubeletConfig{})
 			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find KubeletConfig object %s", initialKey.Name))
-			err = testclient.GetWithRetry(context.TODO(), initialKey, &machineconfigv1.MachineConfig{})
+
+			initialMachineConfigKey := types.NamespacedName{
+				Name:      machineconfig.GetMachineConfigName(profile),
+				Namespace: metav1.NamespaceNone,
+			}
+			err = testclient.GetWithRetry(context.TODO(), initialMachineConfigKey, &machineconfigv1.MachineConfig{})
 			Expect(err).ToNot(HaveOccurred(), fmt.Sprintf("cannot find MachineConfig object %s", initialKey.Name))
 		})
 	})
@@ -597,6 +620,8 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			v1Profile.Name = "v1"
 			v1Profile.ResourceVersion = ""
 			v1Profile.Spec.NodeSelector = map[string]string{"v1/v1": "v1"}
+			v1Profile.Spec.MachineConfigPoolSelector = nil
+			v1Profile.Spec.MachineConfigLabel = nil
 			Expect(testclient.Client.Create(context.TODO(), v1Profile)).ToNot(HaveOccurred())
 
 			key = types.NamespacedName{
@@ -641,6 +666,8 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			v1alpha1Profile.Name = "v1alpha"
 			v1alpha1Profile.ResourceVersion = ""
 			v1alpha1Profile.Spec.NodeSelector = map[string]string{"v1alpha/v1alpha": "v1alpha"}
+			v1alpha1Profile.Spec.MachineConfigPoolSelector = nil
+			v1alpha1Profile.Spec.MachineConfigLabel = nil
 			Expect(testclient.Client.Create(context.TODO(), v1alpha1Profile)).ToNot(HaveOccurred())
 
 			key = types.NamespacedName{
@@ -672,6 +699,8 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 				profile.ResourceVersion = ""
 				profile.Spec.NodeSelector = map[string]string{"withoutNUMA/withoutNUMA": "withoutNUMA"}
 				profile.Spec.NUMA = nil
+				profile.Spec.MachineConfigPoolSelector = nil
+				profile.Spec.MachineConfigLabel = nil
 
 				err = testclient.Client.Create(context.TODO(), profile)
 				Expect(err).ToNot(HaveOccurred(), "Failed to create profile without NUMA")
@@ -702,6 +731,197 @@ var _ = Describe("[rfe_id:27368][performance]", func() {
 			verifyV2V1()
 		})
 	})
+
+	Context("Validation webhook", func() {
+		BeforeEach(func() {
+			if discovery.Enabled() {
+				Skip("Discovery mode enabled, test skipped because it creates incorrect profiles")
+			}
+		})
+
+		validateObject := func(obj runtime.Object, message string) {
+			err := testclient.Client.Create(context.TODO(), obj)
+			Expect(err).To(HaveOccurred(), "expected the validation error")
+			Expect(err.Error()).To(ContainSubstring(message))
+		}
+
+		Context("with API version v1alpha1 profile", func() {
+			var v1alpha1Profile *performancev1alpha1.PerformanceProfile
+
+			BeforeEach(func() {
+				v1alpha1Profile = &performancev1alpha1.PerformanceProfile{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "PerformanceProfile",
+						APIVersion: performancev1alpha1.GroupVersion.String(),
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "v1alpha1-profile",
+					},
+					Spec: performancev1alpha1.PerformanceProfileSpec{
+						RealTimeKernel: &performancev1alpha1.RealTimeKernel{
+							Enabled: pointer.BoolPtr(true),
+						},
+						NodeSelector: map[string]string{"v1alpha1/v1alpha1": "v1alpha1"},
+						NUMA: &performancev1alpha1.NUMA{
+							TopologyPolicy: pointer.StringPtr("restricted"),
+						},
+					},
+				}
+			})
+
+			It("should reject the creation of the profile with overlapping CPUs", func() {
+				reserved := performancev1alpha1.CPUSet("0-3")
+				isolated := performancev1alpha1.CPUSet("0-7")
+
+				v1alpha1Profile.Spec.CPU = &performancev1alpha1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1alpha1Profile, "reserved and isolated cpus overlap")
+			})
+
+			It("should reject the creation of the profile with no isolated CPUs", func() {
+				reserved := performancev1alpha1.CPUSet("0-3")
+				isolated := performancev1alpha1.CPUSet("")
+
+				v1alpha1Profile.Spec.CPU = &performancev1alpha1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1alpha1Profile, "isolated CPUs can not be empty")
+			})
+
+			It("should reject the creation of the profile with the node selector that already in use", func() {
+				reserved := performancev1alpha1.CPUSet("0,1")
+				isolated := performancev1alpha1.CPUSet("2,3")
+
+				v1alpha1Profile.Spec.CPU = &performancev1alpha1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				v1alpha1Profile.Spec.NodeSelector = testutils.NodeSelectorLabels
+				validateObject(v1alpha1Profile, "the profile has the same node selector as the performance profile")
+			})
+		})
+
+		Context("with API version v1 profile", func() {
+			var v1Profile *performancev1.PerformanceProfile
+
+			BeforeEach(func() {
+				v1Profile = &performancev1.PerformanceProfile{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "PerformanceProfile",
+						APIVersion: performancev1.GroupVersion.String(),
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "v1-profile",
+					},
+					Spec: performancev1.PerformanceProfileSpec{
+						RealTimeKernel: &performancev1.RealTimeKernel{
+							Enabled: pointer.BoolPtr(true),
+						},
+						NodeSelector: map[string]string{"v1/v1": "v1"},
+						NUMA: &performancev1.NUMA{
+							TopologyPolicy: pointer.StringPtr("restricted"),
+						},
+					},
+				}
+			})
+
+			It("should reject the creation of the profile with overlapping CPUs", func() {
+				reserved := performancev1.CPUSet("0-3")
+				isolated := performancev1.CPUSet("0-7")
+
+				v1Profile.Spec.CPU = &performancev1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1Profile, "reserved and isolated cpus overlap")
+			})
+
+			It("should reject the creation of the profile with no isolated CPUs", func() {
+				reserved := performancev1.CPUSet("0-3")
+				isolated := performancev1.CPUSet("")
+
+				v1Profile.Spec.CPU = &performancev1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v1Profile, "isolated CPUs can not be empty")
+			})
+
+			It("should reject the creation of the profile with the node selector that already in use", func() {
+				reserved := performancev1.CPUSet("0,1")
+				isolated := performancev1.CPUSet("2,3")
+
+				v1Profile.Spec.CPU = &performancev1.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				v1Profile.Spec.NodeSelector = testutils.NodeSelectorLabels
+				validateObject(v1Profile, "the profile has the same node selector as the performance profile")
+			})
+		})
+
+		Context("with profile version v2", func() {
+			var v2Profile *performancev2.PerformanceProfile
+
+			BeforeEach(func() {
+				v2Profile = &performancev2.PerformanceProfile{
+					TypeMeta: metav1.TypeMeta{
+						Kind:       "PerformanceProfile",
+						APIVersion: performancev2.GroupVersion.String(),
+					},
+					ObjectMeta: metav1.ObjectMeta{
+						Name: "v2-profile",
+					},
+					Spec: performancev2.PerformanceProfileSpec{
+						RealTimeKernel: &performancev2.RealTimeKernel{
+							Enabled: pointer.BoolPtr(true),
+						},
+						NodeSelector: map[string]string{"v2/v2": "v2"},
+						NUMA: &performancev2.NUMA{
+							TopologyPolicy: pointer.StringPtr("restricted"),
+						},
+					},
+				}
+			})
+
+			It("should reject the creation of the profile with overlapping CPUs", func() {
+				reserved := performancev2.CPUSet("0-3")
+				isolated := performancev2.CPUSet("0-7")
+
+				v2Profile.Spec.CPU = &performancev2.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v2Profile, "reserved and isolated cpus overlap")
+			})
+
+			It("should reject the creation of the profile with no isolated CPUs", func() {
+				reserved := performancev2.CPUSet("0-3")
+				isolated := performancev2.CPUSet("")
+
+				v2Profile.Spec.CPU = &performancev2.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				validateObject(v2Profile, "isolated CPUs can not be empty")
+			})
+
+			It("should reject the creation of the profile with the node selector that already in use", func() {
+				reserved := performancev2.CPUSet("0,1")
+				isolated := performancev2.CPUSet("2,3")
+
+				v2Profile.Spec.CPU = &performancev2.CPU{
+					Reserved: &reserved,
+					Isolated: &isolated,
+				}
+				v2Profile.Spec.NodeSelector = testutils.NodeSelectorLabels
+				validateObject(v2Profile, "the profile has the same node selector as the performance profile")
+			})
+		})
+	})
 })
 
 func verifyV1alpha1Conversion(v1alpha1Profile *performancev1alpha1.PerformanceProfile, v1Profile *performancev1.PerformanceProfile) error {
@@ -979,13 +1199,13 @@ func execSysctlOnWorkers(workerNodes []corev1.Node, sysctlMap map[string]string)
 			By(fmt.Sprintf("executing the command \"sysctl -n %s\"", param))
 			out, err = nodes.ExecCommandOnMachineConfigDaemon(&node, []string{"sysctl", "-n", param})
 			Expect(err).ToNot(HaveOccurred())
-			Expect(strings.TrimSpace(string(out))).Should(Equal(expected), fmt.Sprintf("parameter %s value is not %s.", param, expected))
+			Expect(strings.TrimSpace(string(out))).Should(Equal(expected), "parameter %s value is not %s.", param, expected)
 		}
 	}
 }
 
 // execute sysctl command inside container in a tuned pod
-func validatTunedActiveProfile(nodes []corev1.Node) {
+func validateTunedActiveProfile(wrknodes []corev1.Node) {
 	var err error
 	var out []byte
 	activeProfileName := components.GetComponentName(testutils.PerformanceProfileName, components.ProfileNamePerformance)
@@ -997,51 +1217,21 @@ func validatTunedActiveProfile(nodes []corev1.Node) {
 
 	for _, t := range tunedList.Items {
 		if len(t.Spec.Profile) > 0 && t.Spec.Profile[0].Data != nil && strings.Contains(*t.Spec.Profile[0].Data, fmt.Sprintf("include=%s", activeProfileName)) {
-			klog.Warning(fmt.Sprintf("PAO tuned profile amended by '%s' profile, test may fail", t.Name))
+			testlog.Warning(fmt.Sprintf("PAO tuned profile amended by '%s' profile, test may fail", t.Name))
 			if t.Spec.Profile[0].Name != nil {
 				activeProfileName = *t.Spec.Profile[0].Name
 			}
 		}
 	}
 
-	for _, node := range nodes {
-		tuned := tunedForNode(&node)
+	for _, node := range wrknodes {
+		tuned := nodes.TunedForNode(&node, RunningOnSingleNode)
 		tunedName := tuned.ObjectMeta.Name
 		By(fmt.Sprintf("executing the command cat /etc/tuned/active_profile inside the pod %s", tunedName))
 		Eventually(func() string {
 			out, err = pods.ExecCommandOnPod(tuned, []string{"cat", "/etc/tuned/active_profile"})
 			return strings.TrimSpace(string(out))
-		}, testTimeout*time.Second, testPollInterval*time.Second).Should(Equal(activeProfileName),
+		}, cluster.ComputeTestTimeout(testTimeout*time.Second, RunningOnSingleNode), testPollInterval*time.Second).Should(Equal(activeProfileName),
 			fmt.Sprintf("active_profile is not set to %s. %v", activeProfileName, err))
 	}
 }
-
-// find tuned pod for appropriate node
-func tunedForNode(node *corev1.Node) *corev1.Pod {
-	listOptions := &client.ListOptions{
-		Namespace:     components.NamespaceNodeTuningOperator,
-		FieldSelector: fields.SelectorFromSet(fields.Set{"spec.nodeName": node.Name}),
-		LabelSelector: labels.SelectorFromSet(labels.Set{"openshift-app": "tuned"}),
-	}
-
-	tunedList := &corev1.PodList{}
-	Eventually(func() bool {
-		if err := testclient.Client.List(context.TODO(), tunedList, listOptions); err != nil {
-			return false
-		}
-
-		if len(tunedList.Items) == 0 {
-			return false
-		}
-		for _, s := range tunedList.Items[0].Status.ContainerStatuses {
-			if s.Ready == false {
-				return false
-			}
-		}
-		return true
-
-	}, testTimeout*time.Second, testPollInterval*time.Second).Should(BeTrue(),
-		"there should be one tuned daemon per node")
-
-	return &tunedList.Items[0]
-}
