@@ -17,14 +17,13 @@ import (
 	configv1 "github.com/openshift/api/config/v1"
 	tunedv1 "github.com/openshift/cluster-node-tuning-operator/pkg/apis/tuned/v1"
 	mcov1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
-
-	apiextensionsv1beta1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1beta1"
-
 	operatorsv1alpha1 "github.com/operator-framework/operator-lifecycle-manager/pkg/api/apis/operators/v1alpha1"
+	apiextensionsv1 "k8s.io/apiextensions-apiserver/pkg/apis/apiextensions/v1"
 
 	performancev1 "github.com/openshift-kni/performance-addon-operators/api/v1"
 	performancev1alpha1 "github.com/openshift-kni/performance-addon-operators/api/v1alpha1"
 	performancev2 "github.com/openshift-kni/performance-addon-operators/api/v2"
+	testlog "github.com/openshift-kni/performance-addon-operators/functests/utils/log"
 )
 
 var (
@@ -62,7 +61,7 @@ func init() {
 		klog.Exit(err.Error())
 	}
 
-	if err := apiextensionsv1beta1.AddToScheme(scheme.Scheme); err != nil {
+	if err := apiextensionsv1.AddToScheme(scheme.Scheme); err != nil {
 		klog.Exit(err.Error())
 	}
 
@@ -73,13 +72,13 @@ func init() {
 	var err error
 	Client, err = New()
 	if err != nil {
-		klog.Info("Failed to initialize client, check the KUBECONFIG env variable", err.Error())
+		testlog.Info("Failed to initialize client, check the KUBECONFIG env variable", err.Error())
 		ClientsEnabled = false
 		return
 	}
 	K8sClient, err = NewK8s()
 	if err != nil {
-		klog.Info("Failed to initialize k8s client, check the KUBECONFIG env variable", err.Error())
+		testlog.Info("Failed to initialize k8s client, check the KUBECONFIG env variable", err.Error())
 		ClientsEnabled = false
 		return
 	}
@@ -116,9 +115,9 @@ func GetWithRetry(ctx context.Context, key client.ObjectKey, obj runtime.Object)
 	EventuallyWithOffset(1, func() error {
 		err = Client.Get(ctx, key, obj)
 		if err != nil {
-			klog.Infof("Getting %s failed, retrying: %v", key.Name, err)
+			testlog.Infof("Getting %s failed, retrying: %v", key.Name, err)
 		}
 		return err
-	}, 1*time.Minute, 10*time.Second).ShouldNot(HaveOccurred(), "Max numbers of retries reached")
+	}, 1*time.Minute, 10*time.Second).ShouldNot(HaveOccurred(), "Max numbers of retries getting %v reached", key)
 	return err
 }
