@@ -20,20 +20,20 @@ CSV_DIR="${PACKAGE_DIR}/${CSV_VERSION}"
 OUT_ROOT="build/_output"
 OUT_DIR="${OUT_ROOT}/olm-catalog"
 OUT_CSV_DIR="${OUT_DIR}/${PACKAGE_NAME}/${CSV_VERSION}"
-OUT_CSV_FILE="${OUT_CSV_DIR}/${PACKAGE_NAME}.v${CSV_VERSION}.clusterserviceversion.yaml"
+OUT_CSV_FILE="${OUT_DIR}/${CSV_VERSION}/${PACKAGE_NAME}.clusterserviceversion.yaml"
 
 TEMPLATES_DIR="${OUT_ROOT}/templates"
 CSV_TEMPLATE_FILE="${TEMPLATES_DIR}/${PACKAGE_NAME}.v${CSV_VERSION}.clusterserviceversion.yaml"
 ICON_FILE="docs/pao-icon.svg"
 
 EXTRA_ANNOTATIONS=""
-MAINTAINERS=""
+EXTRA_MAINTAINERS=""
 
 if [ -n "$MAINTAINERS_FILE" ]; then
-  MAINTAINERS="-maintainers-from=$MAINTAINERS_FILE"
+  EXTRA_MAINTAINERS="--maintainers-from=$MAINTAINERS_FILE"
 fi
 if [ -n "$ANNOTATIONS_FILE" ]; then
-  EXTRA_ANNOTATIONS="-annotations-from=$ANNOTATIONS_FILE"
+  EXTRA_ANNOTATIONS="--annotations-from=$ANNOTATIONS_FILE"
 fi
 
 clean_package() {
@@ -54,42 +54,48 @@ fi
 clean_package
 
 # do not generate new CRD/CSV for old versions
-if [[ ${CSV_VERSION} =~ 4.6.* ]]; then
+if [[ ${CSV_VERSION} =~ 4.8.* ]]; then
   cp -a deploy/olm-catalog build/_output
 
   # generate a temporary csv we'll use as a template
-  $OPERATOR_SDK generate csv \
-    --operator-name="${PACKAGE_NAME}" \
-    --csv-version="${CSV_VERSION}" \
-    --csv-channel="${CSV_CHANNEL}" \
+  $KUSTOMIZE build config/default | $OPERATOR_SDK generate packagemanifests \
+    --version="${CSV_VERSION}" \
+    --channel="${CSV_CHANNEL}" \
     --default-channel=true \
-    --update-crds \
-    --make-manifests=false \
+    --update-objects \
     --from-version="${CSV_FROM_VERSION}" \
-    --output-dir="${OUT_ROOT}"
+    --deploy-dir="deploy/olm-catalog" \
+    --output-dir="${OUT_DIR}" \
+    --crds-dir="config/crd/bases"
 
-  # copy template CSV file to preserve it for our csv-generator
+  # copy template CSV file to preserve it for our csv-processor
   mv "${OUT_CSV_FILE}" "${CSV_TEMPLATE_FILE}"
 
+  # copy the CRD before the generator will delete it
+  cp "${OUT_DIR}/${CSV_VERSION}/performance.openshift.io_performanceprofiles.yaml" \
+    "${CSV_DIR}/performance.openshift.io_performanceprofiles_crd.yaml"
+
   # using the generated CSV, create the real CSV by injecting all the right data into it
-  build/_output/bin/csv-generator \
+  build/_output/bin/csv-processor \
     --csv-version "${CSV_VERSION}" \
     --operator-csv-template-file "${CSV_TEMPLATE_FILE}" \
     --operator-image "${FULL_OPERATOR_IMAGE}" \
     --olm-bundle-directory "${OUT_CSV_DIR}" \
     --replaces-csv-version "${REPLACES_CSV_VERSION}" \
     --skip-range "${CSV_SKIP_RANGE}" \
+    --min-kube-version "${MIN_KUBE_VERSION}" \
     --icon-from "${ICON_FILE}" \
-    "${MAINTAINERS}" \
-    "${EXTRA_ANNOTATIONS}"
+    "${EXTRA_ANNOTATIONS}" \
+    "${EXTRA_MAINTAINERS}"
 
-  $OPERATOR_SDK generate crds
-  cp -a deploy/crds/performance.openshift.io_performanceprofiles_crd.yaml "${OUT_CSV_DIR}"
+  # restore the deleted CRD
+  cp "${CSV_DIR}/performance.openshift.io_performanceprofiles_crd.yaml" \
+    "${OUT_CSV_DIR}/performance.openshift.io_performanceprofiles_crd.yaml"
+
+  if [[ "$IS_DEV" == true ]]; then
+    # copy generated CSV and CRD back to the repository dir
+    cp "${OUT_CSV_DIR}"/* "${CSV_DIR}/"
+  fi
+
+  echo "New OLM manifests created at ${OUT_DIR}"
 fi
-
-if [[ "$IS_DEV" == true ]]; then
-  # copy generated CSV and CRD back to the repository dir
-  cp "${OUT_CSV_DIR}"/* "${CSV_DIR}/"
-fi
-
-echo "New OLM manifests created at ${OUT_DIR}"
